package caching.dao;

import java.lang.reflect.Proxy;

import caching.logging.DuplicateInvocationReporter;
import caching.logging.InvocationLogger;
import caching.logging.DaoInvocationHandler;

public class DaoFactory {
	private static InvocationLogger logger = InvocationLogger.getInstance();
	static {logger.addObserver(new DuplicateInvocationReporter(1));}
	
	public static DaoFactory getInstance() {
		return new DaoFactory();
	}

	public CustomerDao getCustomerDao() {
		return getCustomerDaoProxy(new CustomerDaoRandomImpl());
	}
	private CustomerDao getCustomerDaoProxy(final CustomerDao dao) {
		return (CustomerDao) Proxy.newProxyInstance(	dao.getClass().getClassLoader(), 
														new Class[] { CustomerDao.class },
														new DaoInvocationHandler<CustomerDao>(dao, logger));
	}

	public static DaoFactory getInstance(InvocationLogger logger) {
		DaoFactory.logger = logger;
		return new DaoFactory();
	}

	public InvocationLogger getLogger() {
		return logger;
	}
}
