package caching.logging;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import caching.cache.Invocation;
import caching.cache.InvocationCache;

public class DaoInvocationHandler<T> implements InvocationHandler {
	private final InvocationCache cache = InvocationCache.getInstance();
	private final InvocationLogger logger;
	private final T underlying;

	public DaoInvocationHandler(T underlying, InvocationLogger logger) {
		this.underlying = underlying;
		this.logger = logger;
	}

	@Override
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		Invocation invocation = new Invocation(method, args);
		Object ret = cache.get(invocation);
		if(ret == null) {
			ret = invokeAndLogInvocation(method, args);
	        cache.put(invocation, ret);
	        return ret;
		} else {
			return ret;
		}
	}

	private Object invokeAndLogInvocation(Method method, Object[] args)	throws IllegalAccessException, InvocationTargetException {
		long before = System.currentTimeMillis();
		Object ret = method.invoke(underlying, args);
		long after = System.currentTimeMillis();
		String fullyQualifiedMethodName = underlying.getClass().getName()+"."+method.getName();
		logger.log(InvocationLogEntry.createEntry(fullyQualifiedMethodName, args, ret, after-before));
		return ret;
	}
}
