package caching.logging;

import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

public class DuplicateInvocationReporter implements Observer {
	private Map<String, Integer> uniqueEntries = new HashMap<String, Integer>();
	private final int duplicationThreshold;
	private boolean hasDuplicatedEntries = false;

	public DuplicateInvocationReporter(int duplicationThreshold) {
		this.duplicationThreshold = duplicationThreshold;
	}

	@Override
	public void update(Observable o, Object arg) {
		String invocationString = ((InvocationLogEntry) arg).getInvocationString();
		if (uniqueEntries.containsKey(invocationString)) {
			Integer newCount = uniqueEntries.get(invocationString) + 1;
			uniqueEntries.put( invocationString, newCount);
			checkThresholdAndReportIfExceeded(newCount, invocationString);
		} else {
			uniqueEntries.put(invocationString, 1);
		}
	}

	private void checkThresholdAndReportIfExceeded(Integer newCount, String invocationString) {
		if (newCount > duplicationThreshold) {
			hasDuplicatedEntries = true;
			reportInvocationString(newCount, invocationString);
		}
	}

	private void reportInvocationString(Integer newCount, String invocationString) {
		System.out.println("Duplicated invocation - count at: "+newCount +" for invocationString: "+invocationString);
	}

	public boolean hasDuplicatedEntries() {
		return hasDuplicatedEntries;
	}

}
