package caching.logging;

import java.util.Arrays;

public class InvocationLogEntry {

	private final String invocationString;
	private final String fullyQualifiedMethodName;
	private final Object[] args;
	private final Object returnValue;
	private final long executionTimeInMilliseconds;
	private final long creationTime;

	protected static InvocationLogEntry createEntry(final String invocationString, final long executionTimeInMilliseconds) {
		return new InvocationLogEntry(invocationString, executionTimeInMilliseconds);
	}
	
	public static InvocationLogEntry createEntry(final String fullyQualifiedMethodName, Object[] args, Object returnValue, long executionTimeInMilliseconds) {
		return new InvocationLogEntry(fullyQualifiedMethodName, args, returnValue, executionTimeInMilliseconds);
	}
	
	private InvocationLogEntry(final String fullyQualifiedMethodName, Object[] args, Object returnValue, long executionTimeInMilliseconds) {
		this.fullyQualifiedMethodName = fullyQualifiedMethodName;
		this.args = args;
		this.returnValue = returnValue;
		this.executionTimeInMilliseconds = executionTimeInMilliseconds;
		this.invocationString = fullyQualifiedMethodName+": "+Arrays.toString(args)+" -> "+returnValue;
		creationTime = System.currentTimeMillis();
	}
	
	private InvocationLogEntry(final String invocationString, final long executionTimeInMilliseconds) {
		this.invocationString = invocationString;
		this.executionTimeInMilliseconds = executionTimeInMilliseconds;
		creationTime = System.currentTimeMillis();
		returnValue = null;
		args = null;
		fullyQualifiedMethodName = null;
	}
	public String getInvocationString() {
		return invocationString;
	}
	public long getExecutionTimeInMilliseconds() {
		return executionTimeInMilliseconds;
	}
	
	public String getFullyQualifiedMethodName() {
		return fullyQualifiedMethodName;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((invocationString == null) ? 0 : invocationString.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		InvocationLogEntry other = (InvocationLogEntry) obj;
		if (invocationString == null) {
			if (other.invocationString != null)
				return false;
		} else if (!invocationString.equals(other.invocationString))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "InvocationLogEntry [args=" + Arrays.toString(args)
				+ ", creationTime=" + creationTime
				+ ", executionTimeInMilliseconds="
				+ executionTimeInMilliseconds + ", fullyQualifiedMethodName="
				+ fullyQualifiedMethodName + ", invocationString="
				+ invocationString + ", returnValue=" + returnValue + "]";
	}

	
}
