package caching.logging;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public class LogAnalyzer {

	private final InvocationLogger logger;

	public LogAnalyzer(final InvocationLogger logger) {
		this.logger = logger;
	}

	public int reportNumberOfEntries() {
		return logger.numberOfEntries();
	}

	public int reportNumberOfUniqueEntries() {
		return reportUniqueInvocationEntries().size();
	}

	public int reportNumberOfOccurencesForEntry(InvocationLogEntry entry) {
		if(logger.contains(entry)) {
			return countEntries(entry);
		} else {
			return 0;
		}
	}

	private int countEntries(InvocationLogEntry entry) {
		Iterator<InvocationLogEntry> iterator = logger.getIterator();
		int count = 0;
		while (iterator.hasNext()) {
			InvocationLogEntry logEntry = (InvocationLogEntry) iterator.next();
			if (entry.equals(logEntry)) {
				count++;
			}
		}
		return count;
	}

	public double reportAverageExceutionTimeByMethodName(String fullyQualifiedMethodName) {
		Iterator<InvocationLogEntry> iterator = logger.getIterator();
		long totalExecutionTime = 0;
		int count = 0;
		while (iterator.hasNext()) {
			InvocationLogEntry logEntry = (InvocationLogEntry) iterator.next();
			if (fullyQualifiedMethodName.equals(logEntry.getFullyQualifiedMethodName())) {
				totalExecutionTime+=logEntry.getExecutionTimeInMilliseconds();
				count++;
			}
		}
		return (totalExecutionTime*1D) / (count*1D);
	}

	public Set<InvocationLogEntry> reportUniqueInvocationEntries() {
		Iterator<InvocationLogEntry> iterator = logger.getIterator();
		HashSet<InvocationLogEntry> uniqueEntries = new HashSet<InvocationLogEntry>();
		while (iterator.hasNext()) {
			InvocationLogEntry entry = (InvocationLogEntry) iterator.next();
			uniqueEntries.add(entry);
		}
		return uniqueEntries;
	}

}
