package caching.cache;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;

import org.junit.Test;

/**
 * Cannot test with Method being anything but null, as mocking a final bootstrapped java class
 * is not possible with jmock. Will instead be tested indirectly when used with the InvocationHandler.
 */
public class TestInvocation {
	@Test
	public void canCreateInvocationFromMethodAndArgs() {
		Invocation invocation = new Invocation(null , new Object[] {});
		assertNotNull(invocation);
	}
	@Test
	public void knowsIfTwoInvocationsAreEqual() {
		Invocation invocationA = new Invocation(null , new Object[] {});
		Invocation invocationB = new Invocation(null , new Object[] {});
		assertEquals(invocationA, invocationB);
	}
	@Test
	public void knowsIfTwoInvocationsWithNullsAreEqual() {
		Invocation invocationA = new Invocation(null , null);
		Invocation invocationB = new Invocation(null , null);
		assertEquals(invocationA, invocationB);
	}
	@Test
	public void givesSameHashCodeForTwoEqualInvocations() {
		Invocation invocationA = new Invocation(null , new Object[] {});
		Invocation invocationB = new Invocation(null , new Object[] {});
		assertEquals(invocationA.hashCode(), invocationB.hashCode());
	}
	@Test
	public void knowsTheSameInvocationToBeEquals() {
		Invocation invocationA = new Invocation(null , new Object[] {});
		assertEquals(invocationA, invocationA);
	}
	@Test
	public void knowsANullInvocationIsNotEqualToAInvocation() {
		Invocation invocationA = new Invocation(null , new Object[] {});
		assertFalse(invocationA.equals(null));
	}
	@Test
	public void knowsAStringIsNotEqualToAInvocation() {
		Invocation invocationA = new Invocation(null , new Object[] {});
		assertFalse(invocationA.equals(""));
	}
	@Test
	public void knowsNotEqualsIfDifferentArgs() {
		Invocation invocationA = new Invocation(null , new Object[] {});
		Invocation invocationB = new Invocation(null , new Object[] {""});
		assertFalse(invocationA.equals(invocationB));
	}
}
