package caching.cache;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.util.Arrays;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Ehcache;
import net.sf.ehcache.Element;
import net.sf.ehcache.config.CacheConfiguration;
import net.sf.ehcache.store.MemoryStoreEvictionPolicy;

import org.junit.Test;

public class TestJCache {
	@Test
	public void testCreateEhcacheViaManager() {
		CacheManager.create();
		CacheManager manager = CacheManager.getInstance();
		System.out.println(Arrays.toString(manager.getCacheNames()));;

		Ehcache cache = manager.getCache("sampleCache2");
		assertNotNull(cache);
		cache.put(new Element("key", "value"));
		assertEquals("Should be value1 that we get with key","value",cache.get("key").getValue());
		
	}
	@Test
	public void testCreateFIFOCacheProgramatically() throws InterruptedException {
		//create and add to manager
		CacheManager singletonManager = CacheManager.create();
		Cache testCache = new Cache(new CacheConfiguration("testCache", 4).overflowToDisk(false).memoryStoreEvictionPolicy(MemoryStoreEvictionPolicy.FIFO));
		singletonManager.addCache(testCache);
		//get from manager and try to put
		//Cache testCache = singletonManager.getCache("testCache");
		testCache.put(new Element("key1", "value1"));
		Thread.sleep(100);
		testCache.put(new Element("key2", "value2"));
		testCache.put(new Element("key3", "value3"));
		testCache.put(new Element("key4", "value4"));
		testCache.put(new Element("key5", "value5"));
		System.out.println(testCache.getKeys());
		assertNull("Should not be value1 that we get with key1 - FIFO should have kicked it out",testCache.get("key1"));
	}
	@Test
	public void testCreateLFUCacheProgramatically() throws InterruptedException {
		//create and add to manager
		CacheManager singletonManager = CacheManager.create();
		Cache lfuCache = new Cache(new CacheConfiguration("testLFUCache", 3).overflowToDisk(false).memoryStoreEvictionPolicy(MemoryStoreEvictionPolicy.LFU));
		singletonManager.addCache(lfuCache);
		//get from manager and try to put
		Cache test = singletonManager.getCache("testLFUCache");
		test.put(new Element("key1", "value1"));
		test.put(new Element("key2", "value2"));
		test.put(new Element("key3", "value3"));
		test.put(new Element("key4", "value4"));
		test.put(new Element("key5", "value5"));
		System.out.println(test.getKeys());
		assertNull("Should not be value1 that we get with key1 - LFU should have kicked it out",test.get("key1"));
		test.get("key4");
		test.get("key4");
		test.get("key3");
		test.put(new Element("key6", "value6"));
		System.out.println(test.getKeys());
		assertNull("Should not be any value with key5 - LFU should have kicked it out",test.get("key5"));
	}
}
