package caching.dao;

import static org.junit.Assert.assertEquals;

import java.io.File;

import org.jmock.Expectations;
import org.jmock.Mockery;
import org.junit.Test;

import caching.logging.LogAnalyzer;

public class TestCallSimulator {
	private static final String LOGFILE_NAME = "C:/Users/Ejer/workspace/caching.prototypes_log_analyser/data/hentProduktoversigtLog.txt";
	private static final String FIRST_LINE_OF_LOG = "jyskeprodukter_da1643-cp1_jb.ind.ebk.produktovs.ear-V2-6-4.log:**	14-01-10	07:59:04,96	hentProduktoversigt	63	8480";
	private static final String LAST_LINE_OF_LOG = "jyskeprodukter_da1643-cp1_jb.ind.ebk.produktovs.ear-V2-6-4.log:**	14-01-10	15:07:05,89	hentProduktoversigt	110	2872";

	@Test
	public void canReadLineFromFile() throws Exception {
		CallSimulator simulator = new CallSimulator(new File(LOGFILE_NAME));
		assertEquals("readline did not give expected result", FIRST_LINE_OF_LOG, simulator.readLine());
	}
	@Test
	public void canReadAllLinesInFile() throws Exception {
		CallSimulator simulator = new CallSimulator(new File(LOGFILE_NAME));
		int numberOfLines = 0;
		while (simulator.readLine() != null) {
			numberOfLines++;
		}
		assertEquals("should be 360 lines in file",360,numberOfLines);
	}
	@Test
	public void canGetUserFromLine() throws Exception {
		CallSimulator simulator = new CallSimulator(new File(LOGFILE_NAME));
		assertEquals("first line should contain user 8480", "8480", simulator.getUserFromLine(FIRST_LINE_OF_LOG));
		assertEquals("last line should contain user 2872", "2872", simulator.getUserFromLine(LAST_LINE_OF_LOG));
	}
	@Test
	public void canGetTimeInMillisecondsFromLine() throws Exception {
		CallSimulator simulator = new CallSimulator(new File(LOGFILE_NAME));
		assertEquals("first line should contain time 1263452344096", 1263452344096L, simulator.getTimeInMillisFromLine(FIRST_LINE_OF_LOG));
	}
	@Test
	public void willCallDaoAsManyTimesAsLogHasEntriesWhenStarted() throws Exception {
		Mockery context = new Mockery();
		final CustomerDao dao = context.mock(CustomerDao.class);
		final CallSimulator simulator = new CallSimulator(new File(LOGFILE_NAME), dao);
		// expectations
        context.checking(new Expectations() {{
        	exactly(360).of (dao).findCustomerById(with(any(String.class)));
        }});
        // execute
        simulator.playLog();
        // verify
        context.assertIsSatisfied();
	}
	@Test
	public void reportsDuplicateInvocationsWhenCallSimulatorCalls() throws Exception {
		DaoFactory daoFactory = DaoFactory.getInstance();
		CallSimulator simulator = new CallSimulator(new File(LOGFILE_NAME) ,daoFactory.getCustomerDao());
		simulator.playLog();
		LogAnalyzer analyzer = new LogAnalyzer(daoFactory.getLogger());
		int numberOfEntries = analyzer.reportNumberOfEntries();
		//removed now that we cache assertEquals("There should be 360 entries", 360, numberOfEntries);
		System.out.println("Number of unique entries: "+analyzer.reportNumberOfUniqueEntries()+" out of total entries: "+numberOfEntries);
	}

	
}
