package caching.dao;

import java.lang.reflect.Proxy;

import org.junit.Test;

import caching.logging.InvocationLogger;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class TestDaoFactory {
	@Test
	public void canCreateDao() {
		DaoFactory factory = DaoFactory.getInstance();
		CustomerDao dao = factory.getCustomerDao();
		assertNotNull(dao);
	}
	@Test
	public void canCreateDynamicProxyDao() {
		DaoFactory factory = DaoFactory.getInstance();
		CustomerDao dao = factory.getCustomerDao();
		assertNotNull(dao);
		assertTrue("dao not instanceof Proxy", dao instanceof Proxy);
	}
	@Test
	public void canProvideReferenceToLoggerWhenCreatedWithout() {
		DaoFactory factory = DaoFactory.getInstance();
		assertNotNull(factory.getLogger());
	}
	@Test
	public void canProvideReferenceToInjectedLogger() {
		InvocationLogger logger = InvocationLogger.getInstance();
		DaoFactory factory = DaoFactory.getInstance(logger);
		assertNotNull(factory.getLogger());
		assertEquals("Logger should be equal", logger, factory.getLogger());
	}
}
