package caching.logging;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.lang.reflect.Method;

import org.junit.Test;

import caching.cache.Invocation;
import caching.cache.InvocationCache;
import caching.dao.CustomerDao;
import caching.dao.Customer;
import caching.dao.DaoFactory;


public class TestDaoInvocationHandler {

	@Test
	public void canLogMethodCallOnUnderlyingObject() {
		InvocationLogger logger = InvocationLogger.getInstance() ;
		CustomerDao dao = DaoFactory.getInstance(logger).getCustomerDao();
		Customer object = dao.findCustomerById("objectId");
		InvocationLogEntry entry = InvocationLogEntry.createEntry("caching.dao.CustomerDaoRandomImpl.findCustomerById: [objectId] -> "+object, 0);
		assertTrue("Logger missing log statement", logger.contains(entry));
	}
	@Test
	public void  canCacheAllDaoInvocations() throws Exception {
		CustomerDao dao = DaoFactory.getInstance().getCustomerDao();
		Customer object = dao.findCustomerById("objectId");
		InvocationCache cache = InvocationCache.getInstance();
		Method method = CustomerDao.class.getDeclaredMethod("findCustomerById", new Class[]{String.class});
		Object objectFromCache = cache.get(new Invocation(method, new Object[] {"objectId"}));
		assertEquals("Original object should be equal to object from Cache", object, objectFromCache);
	}
}
