package caching.logging;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import org.junit.Test;

public class TestInvocationLogEntry {
	@Test
	public void canCreateInvocationLogEntry() {
		InvocationLogEntry entry = InvocationLogEntry.createEntry("invocation string", 1000);
		assertNotNull("createEntry returned null!", entry);
		assertEquals("execution time should be 1000", 1000L, entry.getExecutionTimeInMilliseconds());
	}
	@Test
	public void knowsIfTwoInstancesAreEqual() {
		InvocationLogEntry entry1 = InvocationLogEntry.createEntry("invocation string", 1000);
		InvocationLogEntry entry2 = InvocationLogEntry.createEntry("invocation string", 1000);
		assertEquals("Entry 1 and entry 2 should be equal", true, entry1.equals(entry2));
		assertEquals("Entry 1 and entry 1 should be equal", true, entry1.equals(entry1));
	}
	@Test
	public void knowsIfTwoInstancesAreNotEqual() {
		InvocationLogEntry entry1 = InvocationLogEntry.createEntry("invocation string", 1000);
		InvocationLogEntry entry2 = InvocationLogEntry.createEntry("another string", 1000);
		assertEquals("Entry 1 and entry 2 should not be equal", false, entry1.equals(entry2));
	}
	@Test
	public void knowsThatNullIsNeverEqualToAnotherInstance() {
		InvocationLogEntry entry1 = InvocationLogEntry.createEntry("invocation string", 1000);
		assertEquals("Entry 1 and null not be equal", false, entry1.equals(null));
	}
	@Test
	public void knowsThatAnInstanceIsNotEqualToADifferentObject() {
		InvocationLogEntry entry1 = InvocationLogEntry.createEntry("invocation string", 1000);
		assertEquals("Entry 1 and a String object should not be equal", false, entry1.equals("a string object"));
	}
	@Test
	public void knowsThatAnInstanceIsNotEqualIfTheInvocationstringIsNull() {
		InvocationLogEntry entry1 = InvocationLogEntry.createEntry(null, 1000);
		InvocationLogEntry entry2 = InvocationLogEntry.createEntry("another string", 1000);
		assertEquals("Entry 1 and entry should not be equal", false, entry1.equals(entry2));
	}
	@Test
	public void knowsAHashCodeWhenInvocationStringIsNull() {
		InvocationLogEntry entry1 = InvocationLogEntry.createEntry(null, 1000);
		assertNotNull("Entry 1 should have a hashCode when invocationString is null", entry1.hashCode());

	}
}
