package caching.logging;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.Iterator;
import java.util.Observer;

import org.jmock.Expectations;
import org.jmock.Mockery;
import org.junit.Test;


public class TestInvocationLogger {
	@Test
	public void canLogAnInvocationLogEntry() {
		InvocationLogger logger = InvocationLogger.getInstance();
		InvocationLogEntry entry = InvocationLogEntry.createEntry("log entry", 1000L);
		logger.log(entry);
		assertTrue("Logger doesn't contain new entry: log entry",logger.contains(entry));
	}
	
	@Test
	public void canCreateALogger() {
		InvocationLogger logger = InvocationLogger.getInstance();
		assertNotNull("Failed to get InvocationLogger instance - got null", logger);
	}
	@Test
	public void knowsHowManyEntriesItHas() {
		InvocationLogger logger = InvocationLogger.getInstance();
		logger.log(InvocationLogEntry.createEntry("log entry1", 1000L));
		assertEquals("Logger expected to contain 1 entry", 1, logger.numberOfEntries());
		logger.log(InvocationLogEntry.createEntry("log entry2", 1000L));
		assertEquals("Logger expected to contain 2 entries", 2, logger.numberOfEntries());
	}
	@Test
	public void canProvideIteratorForEntries() {
		InvocationLogger logger = InvocationLogger.getInstance();
		logger.log(InvocationLogEntry.createEntry("entry1", 1000L));
		logger.log(InvocationLogEntry.createEntry("entry2", 1000L));
		Iterator<InvocationLogEntry> iterator = logger.getIterator();
		String allEntries = "";
		while (iterator.hasNext()) {
			allEntries += iterator.next().getInvocationString();
		}
		assertEquals("AllEntries expected to be \"entry1entry2\"","entry1entry2",allEntries);
	}

	@Test
	public void updatesObserversWhenLoggingAnEntry() {
		Mockery context = new Mockery();
		final Observer observer = context.mock(Observer.class);
		
		final InvocationLogger logger = InvocationLogger.getInstance();
		logger.addObserver(observer);
		final InvocationLogEntry entry = InvocationLogEntry.createEntry("invocationString", 0);
		// expectations
        context.checking(new Expectations() {{
            oneOf (observer).update(logger, entry);
        }});
        // execute
        logger.log(entry);
        // verify
        context.assertIsSatisfied();
	}

}
